/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.plugins.grid.definitions

import org.apache.camel.Processor
import org.apache.camel.Exchange
import metridoc.plugins.PluginDB
import metridoc.plugins.grid.GridCreator
import metridoc.utils.AssertState
import metridoc.utils.Assert
import metridoc.camel.utils.ExchangeUtils
import org.apache.camel.util.ExchangeHelper
import metridoc.plugins.impl.grid.GridWrapper
import org.apache.camel.Navigate

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 9/8/11
 * Time: 2:29 PM
 * To change this template use File | Settings | File Templates.
 */
class GridProcessor implements Processor, Navigate<Processor> {

    Processor next
    String type
    Closure filter
    Closure parser
    GridCreator creator
    Map<String, Object> properties

    GridCreator getCreator() {
        if (creator) {
            return creator
        }

        AssertState.notEmpty(type, "type must be specified so the GridCreator can be discovered")

        List<Class<GridCreator>> creators = PluginDB.getInstance().getPlugins("grid")

        creators.each {
            String property = PluginDB.getProperty("type", it)

            if (property == type) {
                creator = it.newInstance()
            }
        }

        Assert.notNull(creator, "Could not find a GridCreator for ${type}")

        return creator
    }

    void process(Exchange exchange) {

        def grid
        if(properties) {
            grid = getCreator().create(exchange, properties)
        } else {
            grid = getCreator().create(exchange)
        }

        exchange.in.body = new GridWrapper(filter: filter, grid: grid, originalExchange: exchange, parser: parser)
        next.process(exchange)
    }

    public List<Processor> next() {
        if (!hasNext()) {
            return null;
        }
        List<Processor> answer = new ArrayList<Processor>(1);
        answer.add(next);
        return answer;
    }

    public boolean hasNext() {
        return next != null;
    }
}
