/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.plugins.impl.grid

import metridoc.plugins.grid.Grid
import org.apache.camel.Exchange
import metridoc.plugins.grid.DefaultGrid

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 9/8/11
 * Time: 4:23 PM
 *
 * Wraps a {@link Grid} implementation to add both a parser and a filter.  The filter is called first and the parser
 * second each time {@link GridWrapper#next} is called.
 */
class GridWrapper extends DefaultGrid {

    /**
     * represents parsing logic to occur after the filter is called. It can accept 1 to three arguments in this order
     * <code>the next value (a {@link Map}), line number and the original exchange that created the grid</code>.  The
     * return value must be a {@link Map}.
     */
    Closure parser
    Closure filter
    Grid grid
    Map<String, Object> next

    private boolean keep() {
        if (filter) {
            boolean keep = metridoc.utils.ClosureUtils.callClosure(filter, next, line, this.originalExchange)
            return keep
        }

        return true
    }

    void remove() {
        grid.remove()
    }

    LinkedHashSet<String> getColumns() {
        grid.columns
    }

    Exchange getExchange() {
        return grid.originalExchange
    }

    int getLine() {
        return grid.line
    }

    public String toString() {
        return "GridWrapper{" +
                "parser=" + parser +
                ", filter=" + filter +
                ", grid=" + grid +
                '}';
    }

    @Override
    Map<String, Object> doNext() {
        if (parser) {
            Map<String, Object> parsedResult = metridoc.utils.ClosureUtils.callClosure(parser, next, line, this.originalExchange)
            next = null
            return parsedResult
        }
        def result = next
        next = null

        return result
    }

    @Override
    boolean doHasNext() {
        if (next != null || grid.hasNext()) {
            if(next == null) {
                next = grid.next()
            }
            boolean keep = keep()

            if(!keep) {
                next = null
                return doHasNext()
            }

            return true
        } else {
            return false
        }
    }
}
