/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.plugins.impl.grid

import metridoc.plugins.grid.Grid

import org.apache.poi.ss.usermodel.Sheet
import org.apache.poi.ss.usermodel.WorkbookFactory
import org.apache.poi.ss.usermodel.Workbook
import org.apache.poi.ss.usermodel.Row
import org.apache.poi.ss.usermodel.Cell
import org.apache.poi.ss.usermodel.DateUtil

import metridoc.plugins.Plugin

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 9/15/11
 * Time: 3:24 PM
 * To change this template use File | Settings | File Templates.
 */
@Plugin(value = "grid", properties = "type=xls")
class XlsGrid extends BaseExcelGrid {

    Sheet sheet
    LinkedHashSet<String> columns

    LinkedHashSet<String> getColumns() {
        if (columns) {
            return columns
        }

        initializeSheetIfNull()
        def row = sheet.getRow(0)
        columns = extractColumns(row, width, hasHeaders)
        if (hasHeaders) {
            rowNum++
        }

        return columns
    }

    private static LinkedHashSet<String> extractColumns(Row row, int width, boolean useFirstRow) {
        def result
        if (useFirstRow) {
            result = extractColumnsFromRow(row)
        } else {
            result = extractDefaultColumns(row)
        }

        if (width) {
            return result.asList().subList(0, width) as LinkedHashSet
        }

        return result
    }

    private static LinkedHashSet<String> extractDefaultColumns(Row row) {
        def columns = new String[row.lastCellNum]
        for (int i = 1; i <= row.lastCellNum; i++) {
            columns[i - 1] = String.valueOf(i)
        }

        return columns
    }

    private static LinkedHashSet<String> extractColumnsFromRow(Row row) {
        def columns = new String[row.lastCellNum]

        for (int i = 0; i < row.lastCellNum; i++) {
            def cell = row.getCell(i)
            if (cell) {
                def cellValue = getCellValue(cell)
                columns[i] = String.valueOf(cellValue)
            } else {
                columns[i] = String.valueOf(i + 1)
            }
        }

        return columns

    }

    private void initializeSheetIfNull() {
        if (sheet == null) {
            Workbook workbook = WorkbookFactory.create(getInputStream())
            if (sheetName) {
                sheet = workbook.getSheet(sheetName)
            } else {
                sheet = workbook.getSheetAt(sheetIndex)
            }
        }
    }



    @Override
    Grid doCreate(InputStream inputStream) {
        def args = [inputStream: inputStream]
        if (parameters) {
            args.putAll(parameters)
        }
        return new XlsGrid(args)
    }

    @Override
    Map<String, Object> doNext() {
        def result = [:]
        Row row = sheet.getRow(rowNum)
        def columnList = getColumns() as List
        for (int i = 0; i < getColumns().size(); i++) {
            result.put(columnList.get(i), getCellValue(row.getCell(i)))
        }

        rowNum++
        return result
    }

    private static Object getCellValue(Cell cell) {

        if (cell == null) {
            return null
        }

        switch (cell.getCellType()) {

            case Cell.CELL_TYPE_STRING:
                return cell.richStringCellValue.getString()
            case Cell.CELL_TYPE_NUMERIC:
                if (DateUtil.isCellDateFormatted(cell)) {
                    return cell.dateCellValue
                } else {
                    return cell.numericCellValue
                }
                break;
            case Cell.CELL_TYPE_BOOLEAN:
                return cell.booleanCellValue
            case Cell.CELL_TYPE_FORMULA:
                return cell.booleanCellValue
            default:
                return null
        }
    }

    @Override
    boolean doHasNext() {
        initializeSheetIfNull()
        sheet.getRow(rowNum) != null
    }
}
