/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.utils

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 9/9/11
 * Time: 11:00 AM
 * To change this template use File | Settings | File Templates.
 */
class ClosureUtils {
    def static callClosure(Closure closure, Object... arguments) {
        def closureToCall = closure
        def parameterTypes = closure.parameterTypes
        def parameterLength = parameterTypes.length
        def argumentSize = arguments.size()
        boolean validSize = parameterLength <= argumentSize

        Assert.isTrue(validSize, "Closure parameters have too many parameters, it has ${parameterLength} and " +
            "needs to be less than or equal to ${argumentSize}")

        for(int i = 0; i < parameterLength; i++) {
            def argument = arguments[i]
            Assert.notNull(argument, "arguments cannot be null")
            closureToCall = closureToCall.curry(arguments[i])
        }

        try {
            return closureToCall()
        } catch (MissingMethodException ex) {
            throw new IllegalArgumentException("There is a missmatch in the types, the closure can accept " +
                "types ${parameterTypes} but trying to pass ${arguments}")
        }
    }
}
